#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include "sendthread.h"
#include "config_file.h"
#include "misc.h"
#include "debug.h"

SendThread::SendThread()
{
	initCurl();
	displayInfos = false;
}

SendThread::~SendThread()
{
	kdebugf();
	if(running())
	{
		kdebugm(KDEBUG_INFO, "Thread is running! Terminating!\n");
		terminate();
		wait();
	}
	cleanup();
}

void SendThread::initCurl()
{
	kdebugf();
	caBundlePath = dataPath("kadu/modules/data/miastoplusa_sms/curl-ca-bundle.crt");
	curl = curl_easy_init();
	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, true);
	curl_easy_setopt(curl, CURLOPT_CAINFO, caBundlePath.data());
	curl_easy_setopt(curl, CURLOPT_AUTOREFERER, TRUE);
	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorMsg);
	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, TRUE);
	curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 10);
	//curl_easy_setopt(curl, CURLOPT_PROXY, "127.0.0.1:8080");
	curl_easy_setopt(curl, CURLOPT_COOKIEFILE, "");
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, (void *)getBody);
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, this);
	setErrorType(NO_ERROR);
	kdebugf2();
}

void SendThread::cleanup()
{
	kdebugf();
	curl_easy_cleanup(curl);
	kdebugf2();
}
	
bool SendThread::performGet(QString path)
{
	kdebugf();
	curl_easy_setopt(curl, CURLOPT_HTTPGET, TRUE);
	curl_easy_setopt(curl, CURLOPT_URL, path.data());
	kdebugm(KDEBUG_INFO, "GET...\n");
	body = "";
	kdebugm(KDEBUG_INFO, "In performGet: errorType = %d\n", errorType);
	if(!curl_easy_perform(curl))
	{
		kdebugm(KDEBUG_INFO, "GET success.\n");
		return true;
	}
	else
	{
		setSuccess(false);
		setErrorType(CONNECTION_ERROR);
		kdebugm(KDEBUG_INFO, "GET FAILED!\n");
		return false;
	}
}

bool SendThread::performPost(QString path, QString postData)
{
	kdebugf();
	curl_easy_setopt(curl, CURLOPT_POST, TRUE);
	curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, postData.length());
	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, postData.data());
	kdebugm(KDEBUG_INFO, "Host...\n");
	curl_easy_setopt(curl, CURLOPT_URL, path.data());
	//kdebugm(KDEBUG_INFO, "postData: %s\n", postData.data());
	body = "";
	if(!curl_easy_perform(curl))
	{
		kdebugm(KDEBUG_INFO, "POST success.\n");
		return true;
	}
	else
	{
		setSuccess(false);
		setErrorType(CONNECTION_ERROR);
		kdebugm(KDEBUG_INFO, "POST FAILED!\n");
		return false;
	}
}

bool SendThread::getSentSMSesInfo()
{
	if(!performGet("http://www.miastoplusa.pl/sms/send_sms.jsp"))
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Getting info about sent messages FAILED!\n");
		setErrorType(UNKNOWN_ERROR);
		return false;
	}
	else
	{
		kdebugm(KDEBUG_INFO, "Got info about sent messages.\n");
		QString temp;
		QString sentSMSesToOthersStr;
		QString numberSubstr;
		QRegExp info(">\\d+<");
		QTextStream s(body, IO_ReadOnly);
		while(!s.atEnd())
		{
			temp = s.readLine();
			if(temp.contains("do innych sieci"))
				sentSMSesToOthersStr = temp;
			
		}

		kdebugm(KDEBUG_INFO, "OthersGSM: %s\n",	sentSMSesToOthersStr.data());
		
		int searchIndex = info.search(sentSMSesToOthersStr);
		kdebugm(KDEBUG_INFO, "searchIndex: %d\n", searchIndex);
		temp = info.cap(0);
		kdebugm(KDEBUG_INFO, "temp: %s", temp.data());
		othersInfos = temp.mid(1, temp.length() - 2);
		kdebugm(KDEBUG_INFO, "othersInfos: %s\n", othersInfos.data());
		
		return true;
	}
}

bool SendThread::login()
{
	kdebugm(KDEBUG_INFO, "post data...\n");
	QString userUrlEncoded = config_file.readEntry("SMS", "MiastoplusaGateway_User");
	userUrlEncoded = curl_escape(userUrlEncoded.data(), userUrlEncoded.length());
	QString passUrlEncoded = config_file.readEntry("SMS", "MiastoplusaGateway_Pass");
        passUrlEncoded = curl_escape(passUrlEncoded.data(), passUrlEncoded.length());
	QString postData = "login=" + userUrlEncoded + "&password=" + passUrlEncoded + "&secureLogin=on";
	if(!performPost("https://www.miastoplusa.pl/auth/LoginCitizen.do", postData))
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Login FAILED!\n");
		return false;
	}
	else
	{
		kdebugm(KDEBUG_INFO, "Login posted.\n");
		return true;
	}
}

bool SendThread::postSMS()
{
	kdebugf();
	QString messageUrlEncoded = curl_escape(msg.data(), msg.length());
	QString targetUrlEncoded = "/sms/send_sms.jsp";
	bool confirmation = config_file.readBoolEntry("SMS", "miastoplusa_sms_request_confirmation", true);
	QString confirmationStr = confirmation ? "30" : "0";
	kdebugm(KDEBUG_INFO, "confirmationStr: %s\n", confirmationStr.data());
	QString postData = "smsType=10&phoneNumber=" + nr + "&message=" + messageUrlEncoded + "&notifyCode=" +
		confirmationStr + "&validity=48&sendDay=-1&sendHour=0&sendMin=0&userId=0&groupId=0"
		"&templateCategory=0&targetURL=" + targetUrlEncoded;
	kdebugm(KDEBUG_INFO, "Posting sms...\n");
	if(!performPost("https://www.miastoplusa.pl/sms/SendSMS2.do", postData))
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Sending sms FAILED!\n");
		return false;
	}
	else
	{
		kdebugm(KDEBUG_INFO, "SMS sent.\n");
		return true;
	}
}

bool SendThread::logout()
{
	kdebugm(KDEBUG_INFO, "Logging out...\n");
	if(!performGet("http://www.miastoplusa.pl/minimal/logout_frameset.jsp"))
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Logging out FAILED!\n");
		return false;
	}
	else
	{
		kdebugm(KDEBUG_INFO, "Logged out.\n");
		return true;
	}
}

size_t getBody(void *buffer, size_t size, size_t nmemb, SendThread *sendThread)
{
	kdebugf();
	char *buf;
	buf = (char *)buffer;
	int bodyLen = nmemb * size;
	sendThread->body.reserve(bodyLen);
	for(int i = 0; i < bodyLen; i++)
		sendThread->body += buf[i];
	
	// ********** DEBUG **********
	/*
	QFile log("/home/patryk/miastoplusa_sms.log");
	if(log.open(IO_WriteOnly | IO_Append))
	{
		kdebugm(KDEBUG_INFO, "Opening log file success.\n");
		QTextStream s(&log);
		s << sendThread->body << flush;
		log.close();
	}
	else
	{
		kdebugm(KDEBUG_INFO, "Opening log file FAILED!\n");
		kdebugm(KDEBUG_INFO, "Because of: %s\n", log.errorString().data());
	}
	*/
	// ********** DEBUG **********
	
	
	return bodyLen;
	kdebugf2();
}

void SendThread::run()
{
	kdebugf();
	sentSMSesToPlus = sentSMSesToOthers = 0;
	setFinished(false);
	setErrorType(NO_ERROR);
	setSuccess();
	
	kdebugm(KDEBUG_INFO, "First GET.\nSUCCESS = %d\n", success);
	// Pierwsze otwarcie strony zebyu odebrac sessionid
	kdebugm(KDEBUG_INFO, "First GET...\n");
	if(!performGet("http://www.miastoplusa.pl/"))
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "First GET failed!\n");
		return;
	}

	kdebugm(KDEBUG_INFO, "Logging in.\nSUCCESS = %d\n", success);

	// Logowanie do bramki
	if(!login())
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Login FAILED!\n");
		return;
	}
	kdebugm(KDEBUG_INFO, "Checking login.\nSUCCESS = %d\n", success);
	if(!validLogin())
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Login FAILED! - wrong login, or password.\n");
		return;
	}
	kdebugm(KDEBUG_INFO, "Posting SMS.\nSUCCESS = %d\n", success);
	
	// Wysylanie smsa
	if(!postSMS())
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Sending SMS FAILED!\n");
		logout();
		return;
	}
	
	kdebugm(KDEBUG_INFO, "Checking if SMS was sent.\nSUCCESS = %d\n", success);
	if(!validSMSSend())
	{
		setFinished();
		kdebugm(KDEBUG_INFO, "Sending SMS FAILED!\n");
		logout();
		return;
	}
	kdebugm(KDEBUG_INFO, "Getting info of sent SMSes.\nSUCCESS = %d\n", success);
	
	// Info o wyslanych sms-ach
	if(displayInfos)
		if(!getSentSMSesInfo())
		{
			setFinished();
			kdebugm(KDEBUG_INFO, "No sent SMSes info.\n");
			logout();
		}
	else
		kdebugm(KDEBUG_INFO, "Not getting infos.\n");
	
	// Wylogowuje
	kdebugm(KDEBUG_INFO, "Logging out.\nSUCCESS = %d\n", success);
	logout();
	kdebugm(KDEBUG_INFO, "SUCCESS = %d\n", success);

	kdebugm(KDEBUG_INFO, "setting finished to TRUE.\n");
	setFinished();
	kdebugf2();
	kdebugm(KDEBUG_INFO, "Emit finished...\n");
	kdebugf2();
}

QString SendThread::getErrorMsg()
{
	kdebugm(KDEBUG_INFO, "isSuccess: %d\nerrorType: %d\n", success, errorType);
	if(isSuccess())
		return QString("");
	QString err(errorMsg);
	QString retMsg;
	//kdebugm(KDEBUG_INFO, "%s\n", errorMsg);
	if(errorType == CONNECTION_ERROR)
	{
		kdebugm(KDEBUG_INFO, "%s\n", err.data());
		if(err.contains("couldn't connect to host"))
			retMsg = tr("Problem with connection to www.miastoplusa.pl!");
		else if(err.contains("SSL certificate problem, verify that the CA cert is OK."))
			retMsg = tr("Certificate verification error!!! Someone is propabely messing up with "
					"you!!! Aborting.") + "\n" + tr("libcurl said:") + "\n" + QString(errorMsg);
		else
			retMsg = tr("Some connection error has occured!") + "\n" + tr("libcurl said:") + "\n" + 
				QString(errorMsg);
	}
	else if(getErrorType() == INVALID_LOGIN)
		retMsg = tr("Login failed! Propabely wrong login name or password. SMS was not sent.");
	else if(getErrorType() == TEN_SMSES_TO_THE_SAME_NUMBER)
		retMsg = tr("Sending SMS failed! Gateway limit of sending 10 messages to the same number reached.");
	else if(getErrorType() == TEMPORALLY_LIMITS)
	{
		QString temp;
		int p, k;
		p = unblockTime.find(";", unblockTime.length() - 30) + 2;
		k = unblockTime.find("<", p) - 1;
		temp = unblockTime.mid(p, k - p + 1);
		kdebugm(KDEBUG_INFO, "p: %d\tk: %d\ttemp: %s\n", p, k, temp.data());
		retMsg = tr("Sending SMS failed! Account temporally blocked due to one of a gateway limits!") + "\n" +
			tr("Account will be unblocked: ") + temp;
	}
	else if(getErrorType() == UNKNOWN_ERROR)
		retMsg = tr("Unknown error has occured while trying to send an SMS.");
	return retMsg;
}

QString SendThread::getInfosMsg()
{
	kdebugf();
	QString retStr = tr("SMSes to other networks left on an account: ") + othersInfos;
	kdebugm(KDEBUG_INFO, "%s\n", retStr.data());
	return retStr;
}

void SendThread::setErrorType(ErrorType err)
{
	kdebugf();
	kdebugm(KDEBUG_INFO, "errorType = %d\n", errorType);
	if(err == NO_ERROR)
		errorType = err;
	else if(errorType == NO_ERROR)
	{
		kdebugm(KDEBUG_INFO, "Changing errorType to: %d\n", err);
		errorType = err;
	}
	kdebugf2();
}

bool SendThread::validLogin()
{
	kdebugf();
	QString invalidLoginSubstr = "Podane haso jest nieprawidowe. Uwaga, wielko liter ma znaczenie.";
	QString temp;
	QTextStream s(body, IO_ReadOnly);
	bool contains = false;
	while(!s.atEnd())
	{
		temp = s.readLine();
		//kdebugm(KDEBUG_INFO, "%s\n", temp.data());
		if(temp.contains(invalidLoginSubstr))
			contains = true;
	}
				
	if(contains)
	{
		kdebugm(KDEBUG_INFO, "Login FAILED!\n");
		setErrorType(INVALID_LOGIN);
		setSuccess(false);
		return false;
	}
	else
	{
		kdebugm(KDEBUG_INFO, "Logged in.\n");
		return true;
	}
	kdebugf2();
}

bool SendThread::validSMSSend()
{
	kdebugf();
	QRegExp invalidSubstr1("Wys.anie\\ SMS-a\\ spowoduje\\ przekroczenie\\ limitu\\ 10\\ SMS-.w\\ na\\ dob.\\ pod\\ ten\\ sam\\ numer");
	//QRegExp invalidSubstr2("*");
	QString unblockSubstr = "Wysyanie zostanie odblokowane";
	unblockSubstr = " zostanie odblokowane";
	QString validSubstr = "SMS przekazany do wysania.";
	QString temp;
	QTextStream s(body, IO_ReadOnly);
	bool containsInvalid1 = false;
	bool containsUnblockSubstr = false;
	bool containsValid = false;
	while(!s.atEnd())
	{
		temp = s.readLine();
		if(temp.contains(validSubstr))
		{
			kdebugm(KDEBUG_INFO, "Contains validSubstr:\n%s\n", temp.data());
			containsValid = true;
		}
		else if(invalidSubstr1.search(temp) > 0)
		{
			kdebugm(KDEBUG_INFO, "Contains invalidSubstr1:\n%s\n", temp.data());
			containsInvalid1 = true;
		}
		else if(temp.contains(unblockSubstr))
		{
			kdebugm(KDEBUG_INFO, "Contains unblockSubstr:\n%s\n", temp.data());
			unblockTime = temp;
			containsUnblockSubstr = true;
		}
	}
	
	if(containsValid)
	{
		kdebugm(KDEBUG_INFO, "SMS sent.\n");
		return true;
	}
	else if(containsInvalid1)
	{
		setErrorType(TEN_SMSES_TO_THE_SAME_NUMBER);
		kdebugm(KDEBUG_INFO, "Sending FAILED!\n\tGateway limit of 10 messages to the same number reached.\n");
		setSuccess(false);
		return false;
	}
	else if(containsUnblockSubstr)
	{
		setErrorType(TEMPORALLY_LIMITS);
		kdebugm(KDEBUG_INFO, "Sending FAILED!\n\tGateway limit of 30 messages per 30 mins reached.\n");
		setSuccess(false);
		return false;
	}
	else
	{
		setErrorType(UNKNOWN_ERROR);
		kdebugm(KDEBUG_INFO, "Unknown error occured during posting SMS!\n");
		setSuccess(false);
		return false;
	}

	// Tutaj kod nie dochodzi, ale kompilator ostrzega wiec
	return false;
	kdebugf2();
}

