/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "smsmiastoplusagatewayslots.h"
#include "smsmiastoplusagateway.h"

#include <qmessagebox.h>

#include "config_dialog.h"
#include "config_file.h"
#include "debug.h"
#include "modules.h"

SmsMiastoplusaGatewaySlots::SmsMiastoplusaGatewaySlots(QObject *parent, const char *name) : QObject(parent, name)
{
	kdebugf();
	
	ConfigDialog::addVGroupBox("SMS", "SMS",
			QT_TRANSLATE_NOOP("@default", "SMS Miasto Plusa Gateway"));
	ConfigDialog::addLineEdit2("SMS", "SMS Miasto Plusa Gateway",
			QT_TRANSLATE_NOOP("@default", "User"));
	ConfigDialog::addLineEdit2("SMS", "SMS Miasto Plusa Gateway",
			QT_TRANSLATE_NOOP("@default", "Pass"));
	ConfigDialog::addCheckBox("SMS", "SMS Miasto Plusa Gateway", 
			QT_TRANSLATE_NOOP("@default", "Request confirmatoin"), "miastoplusa_sms_request_confirmation",
			true);
	ConfigDialog::addCheckBox("SMS", "SMS Miasto Plusa Gateway",
			QT_TRANSLATE_NOOP("@default", "Display infos about sent SMSes"), 
			"miastoplusa_sms_display_infos",
			false);
	
	ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::registerSlotOnClose(this, SLOT(onCloseConfigDialog()));
	ConfigDialog::registerSlotOnApply(this, SLOT(onApplyConfigDialog()));
	kdebugf2();
}

SmsMiastoplusaGatewaySlots::~SmsMiastoplusaGatewaySlots()
{
	kdebugf();
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::unregisterSlotOnClose(this, SLOT(onCloseConfigDialog()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(onApplyConfigDialog()));
	
	ConfigDialog::removeControl("SMS", "Display infos about sent SMSes");
	ConfigDialog::removeControl("SMS", "Request confirmatoin");
	ConfigDialog::removeControl("SMS", "User");
	ConfigDialog::removeControl("SMS", "Pass");
	ConfigDialog::removeControl("SMS", "SMS Miasto Plusa Gateway");
	kdebugf2();
}

void SmsMiastoplusaGatewaySlots::onApplyConfigDialog()
{
	kdebugf();
	QLineEdit *e_miastoplusa_sms_user = ConfigDialog::getLineEdit("SMS", "User");
	QLineEdit *e_miastoplusa_sms_pass = ConfigDialog::getLineEdit("SMS", "Pass");
	config_file.writeEntry("SMS", "MiastoplusaGateway_User", e_miastoplusa_sms_user->text());
	config_file.writeEntry("SMS", "MiastoplusaGateway_Pass", e_miastoplusa_sms_pass->text());
	kdebugf2();
}

void SmsMiastoplusaGatewaySlots::onCloseConfigDialog()
{
	kdebugf();
	modules_manager->moduleDecUsageCount("miastoplusa_sms");
}

void SmsMiastoplusaGatewaySlots::onCreateConfigDialog()
{
	kdebugf();
	QLineEdit *e_miastoplusauser= ConfigDialog::getLineEdit("SMS", "User");
	QLineEdit *e_miastoplusapass= ConfigDialog::getLineEdit("SMS", "Pass");
	e_miastoplusapass->setEchoMode(QLineEdit::Password);
	e_miastoplusauser->setText(config_file.readEntry("SMS", "MiastoplusaGateway_User"));
	e_miastoplusapass->setText(config_file.readEntry("SMS", "MiastoplusaGateway_Pass"));
	modules_manager->moduleIncUsageCount("miastoplusa_sms");
	kdebugf2();
}

SmsGateway* SmsMiastoplusaGatewaySlots::isValidMiastoplusa(const QString& number, QObject* parent)
{
	if(SmsMiastoplusaGateway::isNumberCorrect(number))
		return new SmsMiastoplusaGateway(parent, "sms_miastoplusa_gateway");
	else
		return NULL;
}

SmsMiastoplusaGatewaySlots* sms_miastoplusa_gateway_slots;
